@@ -14,7 +14,9 @@ sub new {
 }
 
 sub cmp {
-    ($_[0]->{ w } || 0) <=> ($_[1]->{ w } || 0);
+    ($_[0]->{ w } || 0) <=> ($_[1]->{ w } || 0) ||
+    ($_[0]->{ u } cmp $_[1]->{ u }) ||
+    ($_[0]->{ u } cmp $_[1]->{ v });
 }
 
 sub val {
